package org.insa.graphs.algorithm.marathon;

import java.util.List;

import org.insa.graphs.algorithm.AbstractSolution.Status;
import org.insa.graphs.algorithm.ArcInspector;
import org.insa.graphs.algorithm.ArcInspectorFactory;
import org.insa.graphs.algorithm.shortestpath.DijkstraAlgorithm;
import org.insa.graphs.algorithm.shortestpath.Label;
import org.insa.graphs.algorithm.shortestpath.ShortestPathAlgorithm;
import org.insa.graphs.algorithm.shortestpath.ShortestPathData;
import org.insa.graphs.algorithm.shortestpath.ShortestPathSolution;
import org.insa.graphs.model.Arc;
import org.insa.graphs.model.Graph;
import org.insa.graphs.model.Node;
import org.insa.graphs.model.Path;

import java.util.Random;


public class MarathonAlgorithm extends ShortestPathAlgorithm {

    protected double pathCost;
    private static final int distanceMarathon = 42195; // en mètres
    public static Random rand = new Random();

    protected MarathonAlgorithm(ShortestPathData data) {
        super(data);
    }

    protected Label createLabel(Node node) {
        return new Label(node);
    }

    @Override
    public ShortestPathSolution run() {
        return (ShortestPathSolution) super.run();
    }

    @Override
    protected ShortestPathSolution doRun() {
        final ShortestPathData data = getInputData();
        final Graph graph = data.getGraph();

        final ShortestPathSolution solution = (new DijkstraAlgorithm(data)).run();
        final Path path = solution.getPath();

        if (!solution.isFeasible())
            return solution;

        while (path.getLength() < distanceMarathon) {
            // Recuperation Arc à supprimer (cacher) et extremites
            final Arc arcToRemove = getLongestArc(path);
            final Node originArcToRemove = arcToRemove.getOrigin();
            final Node destinationArcToRemove = arcToRemove.getDestination();


            // Creations du chemin entre les 2 noeuds dont l'arc a été supprimé
            final ArcInspector arcInspector = new MarathonArcInspector(path, arcToRemove); // filters out cycles and current arc
            final ShortestPathData newData = new ShortestPathData(graph, originArcToRemove, destinationArcToRemove, arcInspector);
            final Path newPath = (new DijkstraAlgorithm(newData)).run().getPath();
            
            // could get stuck in a loop if no path is found, given the current arcToRemove strategy
            if (newPath != null) {
                // Ajout du path trouvé à l'indice où on l'a enlevé
                path.replaceArc(arcToRemove, newPath.getArcs());
            }
        }
        
        return solution;
    }

    

    public double getCostPath() {
        return this.pathCost;
    }

    public static int getDistance() {
        return distanceMarathon;
    }

    private Arc getLongestArc(Path path) {
        // TODO : A MODIFIER POUR AMELIORER LA COHERENCE DU CHEMIN
        final List<Arc> arcs = path.getArcs();
        // we checked that the path is feasible in run
        // so there should be at least one Arc (unless start = destination ?)
        if (arcs.size() == 0)
            return null;

        Arc longestArc = arcs.get(0);
        for (Arc arc : arcs) {
            if (arc.getLength() > longestArc.getLength()) {
                longestArc = arc;
            }
        }
        return arcs.get(Math.abs(rand.nextInt() % (arcs.size())));

        //return longestArc;
        //int indiceArcToRemove = 0; //Math.abs(rand.nextInt() % (longueurPath - 1));
    }
}

