package org.insa.graphs.algorithm.shortestpath;

import org.insa.graphs.model.Node;
import org.insa.graphs.model.Point;
import org.insa.graphs.algorithm.AbstractInputData.Mode;
import org.insa.graphs.model.RoadInformation;

public class LabelStar extends Label {
  private double distanceToDestination;
  private int MaximumSpeed;
  private Node destination;

  public LabelStar(Node node, int MaximumSpeed, Node destination) {
    super(node);
    this.MaximumSpeed = MaximumSpeed;
    this.distanceToDestination = -1;
    this.destination = destination;
  }

  private void initializeDistance() {
    // precision was never an answer
    if (this.MaximumSpeed < 0) {
      distanceToDestination = (double) Point.distance(node.getPoint(), this.destination.getPoint());
    }
    else {
      distanceToDestination = (double) Point.distance(node.getPoint(), this.destination.getPoint()) / (1000 * MaximumSpeed);
    }
  }

  @Override
  public double getTotalCost() {
    if (this.distanceToDestination < 0)
      this.initializeDistance();
    return this.getCost() + this.distanceToDestination;
  }

  @Override
  /*
   * This override of the function CompareTo addresses the case:
   * "En cas d'égalité, on considèrera en premier le sommet ayant le plus petit coût estimé à la destination."
   */
  public int compareTo(Label other) {
    final double difference = this.getTotalCost() - other.getTotalCost();
    int retour =  (int) Math.signum(difference);
    if (Math.abs(difference) < 0.01) {
      // En cas d'égalité:
      // Récupérer le sommet avec le plus petit coût estimé à la destination
      // <=> Récupérer le sommet avec le plus grand coût depuis l'origine
      if ((this.getCost() - other.getCost() > 0.0)) {
        retour = 1;
      }
      else{
        retour = -1;
      }
    }
    return retour;
  }
}
